@@ -27,7 +27,18 @@ def from_rainbow_FITS(rainbow, filepath):
     hdu_list = fits.open(filepath)
 
     # load the header into metadata
-    rainbow.metadata = dict(hdu_list["primary"].header)
+    h = hdu_list["primary"].header
+    for k in h:
+        if k.lower() in [
+            "name",
+            "wscale",
+            "tscale",
+            "signal_to_noise",
+            "history",
+        ]:  # KLUDGE!
+            rainbow.metadata[k.lower()] = h[k]
+        elif k not in ["SIMPLE", "BITPIX", "NAXIS", "EXTEND"]:
+            rainbow.metadata[k] = h[k]
 
     # load into the three core dictionaries
     for e in ["fluxlike", "wavelike", "timelike"]:
