@@ -1,6 +1,9 @@
 from .colors import *
 from .imshow import *
-from .plot import *
+from .plot_lightcurves import *
 from .animate import *
 from .diagnostics import *
 from .interactive import *
+from .plot_spectra import *
+from .plot import *
+from .utilities import *
